#Copyright Massachusetts Institute of technology, 1998.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

use strict;
require Exporter;
require AutoLoader;

use GDBM_File;
use W3C::Database::DBStreamHandle;

$W3C::Database::GdbmInterface::REVISION = '$Id: GdbmInterface.pm,v 1.11 1999/12/14 13:14:39 eric Exp $ ';

package W3C::Database::GdbmInterface;
use vars qw($VERSION $DSLI @ISA @EXPORT @EXPORT_OK @TODO);
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = 0.97;
$DSLI = 'adpO';
@TODO = ();

package W3C::Database::GdbmConn;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    bless ($self, $class);
    my %db;
    $self->{DB} = \%db;
    $self->{NAME} = shift;
    $self->{MODE} = shift;
    $self->{MODE} = $W3C::DBStreamHandle::READ_WRITE if (!defined $self->{MODE});
    if ($self->{MODE} == $W3C::DBStreamHandle::READ_ONLY) {
	tie(%db, 'GDBM_File', $self->{NAME}, &GDBM_File::GDBM_READER, 0644) || die 'couldn\'t open "'.$self->{NAME}.'" : '.$!;
    } elsif ($self->{MODE} == $W3C::DBStreamHandle::WRITE_ONLY) {
	if (-e $self->{NAME}) {
	    tie(%db, 'GDBM_File', $self->{NAME}, &GDBM_File::GDBM_WRITER, 0644) || die 'couldn\'t open "'.$self->{NAME}.'" : '.$!;
	} else {
	    tie(%db, 'GDBM_File', $self->{NAME}, &GDBM_File::GDBM_WRCREAT,0644) || die 'couldn\'t open "'.$self->{NAME}.'" : '.$!;
	}
    } elsif ($self->{MODE} == $W3C::DBStreamHandle::CREATE) {
	tie(%db, 'GDBM_File', $self->{NAME}, &GDBM_File::GDBM_WRCREAT,0644) || die 'couldn\'t open "'.$self->{NAME}.'" : '.$!;
    } else {
	die "open what gdbm file with what access?";
    }
    return $self;
}

sub getDB {
    my $self = shift;
    return $self->{DB};
}

package W3C::Database::GdbmInterface;

#####
# per-object data

#####
# new - prepare a W3C::Database::GdbmInterface with a new connection

sub newConnection {
    return new W3C::Database::GdbmConn(@_);
}

package W3C::Database::GDBMHandle;

@W3C::Database::GDBMHandle::ISA = ('W3C::Database::DelimStreamHandle'); # grab encoding methods

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($gdbmFile) = (shift);
    my $self  = $class->SUPER::new(@_);
    bless ($self, $class);
    my $gdbmConn = W3C::Database::GdbmInterface::newConnection($gdbmFile, $self->{ACCESS});
    $self->{DB} = $gdbmConn->getDB;
    return $self;

}

sub put {
    my ($self, @values) = @_;
    $self->{DB}{$values[0]} = $self->encode(@values[1..$#values]);
}

sub forEach {
    my ($self) = @_;
    foreach my $key (keys %{$self->{DB}}) {
	my $value = $self->{DB}{$key};
	$self->{SINK}->put($key, $self->decode($value));
    }
}

package W3C::Database::GdbmInterface;

1;

__END__

=head1 NAME

W3C::Database::GdbmInterface - DBStreamHandle to talk to gdbm databases

=head1 SYNOPSIS

see ddb.pl

=head1 DESCRIPTION

This module is part of the W3C::Database CPAN module.

=head1 AUTHOR

Eric Prud\'hommeaux <eric@w3.org>

=head1 SEE ALSO

perl(1).

=cut
